package org.expeditee.agents;

import org.expeditee.gui.AttributeUtils;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameGraphics;
import org.expeditee.gui.FrameIO;
import org.expeditee.items.Item;
import org.expeditee.items.Text;

public class ComputeTree extends DefaultAgent {

	@Override
	protected Frame process(Frame frame) {
		if (computeFrame(frame) == null) {
			message("Nothing to compute!");
		}

		return null;
	}

	/**
	 * Computes the value of a frame.
	 * 
	 * @param frame
	 * @return the value of the frame or null if the frame does not have the
	 *         compute tag or any values to compute;
	 */
	private Double computeFrame(Frame frame) {
		// Search for @Compute frame tag
		String computeTag = null;
		for (Item i : frame.getItems()) {
			if (i.isAnnotation()) {
				String s = ((Text) i).getFirstLine().toLowerCase().trim();
				if (s.startsWith("@compute:")) {
					computeTag = s;
					break;
				}
			}
		}

		// Check that the compute tag exists
		if (computeTag == null)
			return null;

		// Iterate through all valid items on the frame performing the correct
		// operation
		char operator = computeTag.charAt(computeTag.length() - 1);
		// double defaultValue = (operator == '*' ? 1.0 : 0.0);
		Double result = null;

		for (Item i : frame.getBodyTextItems(false)) {

			Double value = null;
			// Process the frame that each item is linked to
			if (i.getLink() != null) {
				value = computeFrame(FrameIO.LoadFrame(i.getAbsoluteLink()));
				// Set the items text
				if (value != null)
					AttributeUtils.setSingleValue((Text) i, value.toString());
			}

			if (value == null) {
				value = AttributeUtils.getDoubleValue(((Text) i)
						.getTextNoList());
			}

			if (value != null) {
				_itemCount++;

				// the first time we find an item init result
				if (result == null)
					result = (operator == '*' ? 1.0 : 0.0);

				result = ((operator == '*') ? (result * value)
						: (result + value));
			}
		}

		if (result != null)
			AttributeUtils.setSingleValue(frame.getTitle(), result.toString());
		_frameCount++;
		FrameIO.ForceSaveFrame(frame);
		
		return result;
	}
}
