package org.expeditee.io;

import java.io.IOException;
import java.util.List;

import org.expeditee.gui.Frame;
import org.expeditee.items.Item;
import org.expeditee.items.Picture;
import org.expeditee.items.Text;

public class HTMLBWriter extends DefaultTreeWriter {

	private int _indent = 0;

	private static final String INDENT = "   ";

	@Override
	protected void initialise(Frame start) throws IOException {
		_format = ".html";
		super.initialise(start);

		_writer.write("<HTML>" + ItemWriter.NEW_LINE);
		_writer.write("<HEAD>" + ItemWriter.NEW_LINE);
		_writer.write("</HEAD>" + ItemWriter.NEW_LINE);
		_writer.write("<BODY>" + ItemWriter.NEW_LINE);
	}

	@Override
	protected String finaliseTree() throws IOException {
		_writer.write("</BODY>" + ItemWriter.NEW_LINE);
		return super.finaliseTree();
	}

	@Override
	protected void writeStartLink(Item linker) throws IOException {
		_indent++;
	}

	@Override
	protected void writeEndLink(Item linker) throws IOException {
		_indent--;
	}

	@Override
	protected void writeTitle(Text toWrite, List<Item> items)
			throws IOException {
		String heading = toWrite.getTextNoList();
		String tag = "H" + (_indent + 1);
		// indenting for tag
		indent();
		_writer.write("<" + tag + ">" + heading + "</" + tag + ">"
				+ ItemWriter.NEW_LINE);
	}

	@Override
	protected void writeText(Text text) throws IOException {
		indent();
		_writer.write("<p>");
		_writer.write(text.getTextNoList().replace('\n', ' '));
		_writer.write("</p>" + ItemWriter.NEW_LINE);
	}

	private void indent() throws IOException {
		for (int i = 0; i < _indent; i++)
			_writer.write(INDENT);
	}

	@Override
	protected void writePicture(Picture pic) throws IOException {
		Text source = pic.getText();

		String line = source.getFirstLine();
		line = line.substring(line.indexOf(":") + 1).trim();

		indent();

		_writer.write("<img src = '../images/" + line + "'>");
		_writer.write(ItemWriter.NEW_LINE);
	}
}
