package org.expeditee.items;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Image;
import java.awt.image.ImageObserver;
import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JFrame;

import org.expeditee.gui.DisplayIO;
import org.expeditee.gui.Frame;

//Static methods that provide functions for the objects\
//mostly to transform values (string -> color etc).

/**
 * Static methods that provide functions for use in Items.
 */
public class ItemUtils {
	private static JFrame _jf = new JFrame();

	// Tag constants
	public static final int TAG_SORT = 0;

	public static final int TAG_JOIN = 1;

	public static final int TAG_INDENT = 2;

	public static final int TAG_OVERLAY = 3;

	public static final int TAG_ACTIVE_OVERLAY = 4;

	public static final int TAG_IMAGE = 5;

	public static final int TAG_ITEM_TEMPLATE = 6;

	public static final int TAG_ANNOTATION_TEMPLATE = 7;

	public static final int TAG_CODE_COMMENT_TEMPLATE = 8;

	public static final int TAG_MENU = 9;

	public static final int TAG_MENU_NEXT = 10;

	public static final int TAG_PARENT = 11;

	public static final int TAG_LITERAL = 12;

	public static final int TAG_FRAME_IMAGE = 13;

	public static final int TAG_BACKUP = 14;

	public static final int TAG_POINTTYPE = 15;

	public static final int TAG_MIN = 0;

	public static final int TAG_MAX = 15;
	
	// Brook: Im claiming this number!
	public static final int TAG_IWIDGET = 16;
	

	/**
	 * Determines if the given List of Items contains an Item that is one of the
	 * pre-defined tags.
	 * 
	 * @param items
	 *            The list of Items to search through
	 * @param tag
	 *            The Tag to search for, this should correspond to one of the
	 *            predefined constants in this class
	 * @return True if an Item was found that is the given Tag, False otherwise.
	 */
	public static boolean ContainsTag(List<Item> items, int tag) {
		return ContainsTag(items, GetTag(tag));
	}

	public static boolean ContainsTag(List<Item> items, String tag) {
		return (FindTag(items, tag) != null);
	}

	/**
	 * Searches the given List of Items for an Item that is one of the
	 * pre-defined tags.
	 * 
	 * @param items
	 *            The list of Items to search through
	 * @param tag
	 *            The Tag to search for, this should correspond to one of the
	 *            predefined constants in this class
	 * @return The Item that is the given tag if one is found, or False if none
	 *         is found
	 */
	public static Text FindTag(List<Item> items, int tag) {
		return FindTag(items, GetTag(tag));
	}

	/**
	 * Searches the given List of Items for an Item that is the given tag
	 * 
	 * @param items
	 *            The list of Items to search through
	 * @param toFind
	 *            The Tag to search for, this should include the at (@) symbol
	 * @return The Item that is the given tag if one is found, or False if none
	 *         is found
	 */
	public static Text FindTag(List<Item> items, String toFind) {
		for (Item i : items) {
			if (i instanceof Text && i.isAnnotation())
				if (((Text) i).startsWith(toFind))
					return (Text) i;
		}

		return null;
	}

	/**
	 * Determines if the given Item is one of the pre-defined tags in this class
	 * 
	 * @param toCheck
	 *            The Item to check
	 * @param tag
	 *            The tag to check the Item against, this should correspond to
	 *            one of the constants defined in this class
	 * @return True if the Item matches the given tag, false otherwise
	 */
	public static boolean isTag(Item toCheck, int tag) {
		/*
		 * if( !(toCheck instanceof Text) || !toCheck.isAnnotation()) return
		 * false;
		 * 
		 * Text txt = (Text) toCheck; //tags are ase-insensitive
		 * if(String.CASE_INSENSITIVE_ORDER.compare(txt.getFirstLine(),GetTag(tag)) ==
		 * 0 || txt.startsWith(GetTag(tag) + " ")) return true;
		 * 
		 * return false;
		 */
		return isTag(toCheck, GetTag(tag));
	}

	/**
	 * Checks if the given Item contains the desired tag (case insensitive)
	 * 
	 * @param toCheck
	 *            The Item to check for the given tag
	 * @param tag
	 *            The tag to check for in the given Item
	 * @return True if the tag is found in the given Item, False otherwise.
	 */
	public static boolean isTag(Item toCheck, String tag) {
		if (!(toCheck instanceof Text) || !toCheck.isAnnotation())
			return false;

		Text txt = (Text) toCheck;
		// tags are ase-insensitive
		if (String.CASE_INSENSITIVE_ORDER.compare(txt.getFirstLine().trim(), tag) == 0
				|| txt.startsWith(tag + " ", true))
			return true;

		return false;
	}

	/**
	 * Strips off the given tag from the given String, and returns wathever is
	 * left
	 * 
	 * @param toStrip
	 *            The String to strip the Tag from
	 * @param tag
	 *            The tag to remove from the String
	 * @return The String that results from removing the given Tag from the
	 *         given String, or null if the given String is not the given Tag
	 */
	public static String StripTag(String toStrip, String tag) {
		if (toStrip.toLowerCase().startsWith(tag.toLowerCase())) {
			toStrip = toStrip.substring(tag.length()).trim();
			return toStrip;
		}

		/**
		 * TODO: Change this to use REGEX
		 */

		return null;
	}

	/**
	 * The same as StripTag(String, String), but this method iterates through
	 * the list of pre-defined tags until one is found.
	 * 
	 * @param toStrip
	 *            The String to strip the tag from
	 * @return The String that results from stripping off the Tag, or null if
	 *         the given String was not a tag
	 */
	public static String StripTag(String toStrip) {
		// there must be something left after stripping
		if (toStrip == null)// || toStrip.trim().indexOf(" ") < 0)
			return toStrip;

		for (int i = TAG_MIN; i <= TAG_MAX; i++) {
			String res = StripTag(toStrip, GetTag(i));
			if (res != null)
				return res;
		}

		return null;
	}

	/**
	 * Strips the first character from a string if it is the @ symbol
	 * @param toStrip
	 *            the string to be stripped
	 * @return the stripped version of the string
	 */
	public static String StripTagSymbol(String toStrip) {
		// there must be something left after stripping
		if (toStrip != null) {
			if (toStrip.length() > 0) {
				if (toStrip.charAt(0) == '@') {
					return toStrip.substring(1);
				}
			}
		}

		return toStrip;
	}

	/**
	 * Converts the given int to the String tag. The int should correspond to
	 * one of the constants in this class, if it does not this method will
	 * return null.
	 * 
	 * @param tag
	 *            The int corresponding to the constants in this class of which
	 *            tag to return
	 * @return The String representation of the given Tag, or null if the given
	 *         value does not have a tag associated
	 */
	public static String GetTag(int tag) {
		switch (tag) {
		case TAG_SORT:
			return "@sort";
		case TAG_JOIN:
			return "@join";
		case TAG_INDENT:
			return "@indent";
		case TAG_OVERLAY:
			return "@o";
		case TAG_ACTIVE_OVERLAY:
			return "@ao";
		case TAG_IMAGE:
			return "@i:";
		case TAG_ITEM_TEMPLATE:
			return "@itemtemplate";
		case TAG_ANNOTATION_TEMPLATE:
			return "@annotationtemplate";
		case TAG_CODE_COMMENT_TEMPLATE:
			return "@commenttemplate";
		case TAG_MENU:
			return "@menu";
		case TAG_MENU_NEXT:
			return "@nextmenu";
		case TAG_PARENT:
			return "@parent";
		case TAG_LITERAL:
			return "@lit";
		case TAG_FRAME_IMAGE:
			return "@f";
		case TAG_BACKUP:
			return "@old";
		case TAG_POINTTYPE:
			return "@pointtype:";
		case TAG_IWIDGET:
			return "@iw";
		default:
			return null;
		}
	}

	/**
	 * Returns a FontMetrics object that corresponds to the given Font.
	 * 
	 * @param font
	 *            The Font to derive the FontMetrics from
	 * @return FontMetrics that correspond to the given Font
	 */
	public static FontMetrics getFontMetrics(Font font) {
		return _jf.getFontMetrics(font);
	}

	public static Image CreateImage(int x, int y) {
		return _jf.createImage(x, y);
	}

	/**
	 * Creates a picture object from the information stored in the given Text
	 * object. <br>
	 * The paths searched are in the following order:<br>
	 * /images/<br>
	 * the source text as a relative path (from program root folder). <br>
	 * the source text as an absolute path <br>
	 * <br>
	 * If the Image file cannot be found on disk null is returned.
	 * 
	 * @param source
	 *            The Text file containing the Picture infomation
	 * @return The Picture object representing the file, or Null if the file is
	 *         not found.
	 */
	public static Picture CreatePicture(Text source, ImageObserver observer) {
		String text = source.getTextNoList();
		String path = "";
		String size = "";

		try {
			// remove @i tag
			text = text.replaceFirst("@i:", "");
			text = text.replaceAll("\n", "");
			text = text.trim();

			if (text.indexOf(".") >= text.lastIndexOf(" "))
				path = text;
			else
				path = text.substring(0, text.lastIndexOf(" "));

			size = text.substring(path.length()).trim();

			// try images subdirectory
			File file = null;

			for (String dir : org.expeditee.gui.UserSettings.ImageDirs) {
				file = new File(dir + path);
				if (file.exists() && !file.isDirectory())
					break;
			}

			if (file == null || !file.exists() || file.isDirectory())
				file = new File(path);

			// try relative path
			if (!file.exists() || file.isDirectory()) {
				URL picture = new Object().getClass().getResource(path);

				// decode to remove %20 in windows folder names
				if (picture != null) {
					try {
						path = URLDecoder.decode(picture.getFile(), "UTF-8");
					} catch (UnsupportedEncodingException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}

			} else
				path = file.getPath();

			// if the image isn't found by now, give up.
			file = new File(path);
			if (!file.exists() || file.isDirectory()) {
				return null;
			}

		} catch (Exception e) {
			return null;
		}

		try {
			Picture pic = new Picture(source, path, size, observer);

			return pic;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

	}

	public static Picture CreateFramePicture(Text source, ImageObserver observer) {
		String size = source.getFirstLine();

		// remove @f tag
		size = size.replaceFirst("@f", "");
		size = size.trim();

		try {
			Picture pic = new Picture(source, null, size, observer);
			return pic;
		} catch (Exception e) {
			//e.printStackTrace();
			return null;
		}
	}

	/**
	 * Creates a deep copy of the given List of Items.
	 * 
	 * @param toCopy
	 *            The list of Items to copy
	 * @return A list containing a copy of all Items in the given List
	 */
	public static List<Item> CopyItems(List<Item> toCopy) {
		return CopyItems(toCopy, false);
	}
	public static List<Item> CopyItems(List<Item> toCopy, boolean extrude) {
		// The copies to return
		List<Item> copies = new ArrayList<Item>();

		// list of dots at the end of lines
		List<Item> lineEnds = new ArrayList<Item>();
		List<Line> lines = new ArrayList<Line>();

		List<Item> vConst = new ArrayList<Item>();
		List<Item> hConst = new ArrayList<Item>();

		List<Item> seen = new ArrayList<Item>();
		
		// Widgets are super special
		List<InteractiveWidget> widgets = new ArrayList<InteractiveWidget>();
		
		for (Item i : toCopy) {
			
//			BROOK
			if (i instanceof WidgetCorner) { // dont add these 

				if (!widgets.contains(((WidgetCorner)i).getWidgetSource()))
					widgets.add(((WidgetCorner)i).getWidgetSource());
				
//			BROOK
			} else if (i instanceof WidgetEdge) { // dont add these 

			// lines are recreated later
			} else if (i instanceof Line) {
				Line line = (Line) i;
				lines.add(line);

				lineEnds.add(line.getEndItem());
				lineEnds.add(line.getStartItem());

				List<Constraint> consts = line.getStartItem().getConstraints();
				if (consts != null) {
					for (Constraint c : consts) {
						if (c.contains(line.getEndItem())) {
							if (c.getConstraintType() == Constraint.VERTICAL) {
								vConst.add(line.getStartItem());
								vConst.add(line.getEndItem());
							} else {
								hConst.add(line.getStartItem());
								hConst.add(line.getEndItem());
							}
						}
					}
				}
			} else {
				seen.add(i);
			}
		}

		// only dots that are not line ends are copied
		seen.removeAll(lineEnds);

		// copy all single dots
		for (Item i : seen) {
			Item copy = i.copy();

			if (i.getParent() != null) {
				// if this is the frame name, make sure the frame is saved (in
				// case it is a TDFC frame)
				if (i == i.getParent().getFrameNameItem())
					i.getParent().setChanged(true);

				// if this is the title of the frame, link it to the frame
				if (i.getLink() == null && i == i.getParent().getTitle()
						&& toCopy.size() == 1) {
					// save the frame after copying
					i.getParent().setChanged(true);
					copy.setLink(i.getParent().getFrameName());
				}
			}

			copies.add(copy);
		}

		seen.clear();

		// replace line ends with their copies
		// this is done here so that copied lines can still share end points
		for (Item i : lineEnds) {
			// if a copy of this dot has not yet been created
			// only copy dots that are in the list of dots to copy, leave
			// others
			// untouched
			if (!seen.contains(i) && toCopy.contains(i)) {
				// create a copy of the line end
				Item copy = i.copy();

				//copy.setID(DisplayIO.getCurrentFrame().getNextItemID());

				copy.removeAllLines();
				copy.removeAllConstraints();
				
				// replace all other instances with the copy
				ReplaceAll(lineEnds, i, copy);
				ReplaceAll(vConst, i, copy);
				ReplaceAll(hConst, i, copy);
				
				if (extrude) {
					Frame frame = DisplayIO.getCurrentFrame();
					Line newLine = new Line(i,copy, frame.getNextItemID());
					frame.addItem(newLine);
					copies.add(newLine);
				}

				seen.add(i);
				copies.add(copy);
			}
		}

		// recreate lines
		for (int i = 0; i < lineEnds.size(); i += 2) {
			Line line = (Line) lines.get(i / 2).copy();

			line.setID(DisplayIO.getCurrentFrame().getNextItemID());
			line.setStartItem(lineEnds.get(i));
			line.setEndItem(lineEnds.get(i + 1));

			copies.add(line);
		}

		// recreate vertical constraints
		for (int i = 0; i < vConst.size(); i += 2) {
			new Constraint(vConst.get(i), vConst.get(i + 1), DisplayIO
					.getCurrentFrame().getNextItemID(), Constraint.VERTICAL);
		}

		// recreate horizontal constraints
		for (int i = 0; i < hConst.size(); i += 2) {
			new Constraint(hConst.get(i), hConst.get(i + 1), DisplayIO
					.getCurrentFrame().getNextItemID(), Constraint.HORIZONTAL);
		}

		// BROOK
		for (InteractiveWidget iw : widgets) {
			try {
				
				InteractiveWidget icopy = iw.copy();
				copies.addAll(icopy.getItems());
				
			} catch (InteractiveWidgetNotAvailableException e) {
				e.printStackTrace();
			}
		}
		
		return copies;
	}

	private static void ReplaceAll(List<Item> list, Item replace, Item with) {
		// replace all other instances with the copy
		for (int pos = 0; pos < list.size(); pos++)
			if (list.get(pos) == replace) {
				list.set(pos, with);
			}
	}

	/**
	 * Attempts to create a new line that starts from the given Item
	 * ('unreeling'). The Item must already have at least one line, and not be a
	 * line itself to be unreeled from.
	 * 
	 * @param toUnreelFrom
	 *            The Item that will be one end point of the new line
	 * @return A List containing the newly created Item and Line that unreel
	 *         from the given Item, or null if this Item cannot be unreeled
	 *         from.
	 */
	public static List<Item> UnreelLine(Item toUnreelFrom) {
		// the Item must already have one line to be unreeled from
		if (toUnreelFrom == null || toUnreelFrom.getLines().size() < 1)
			return null;

		List<Item> unreel = new ArrayList<Item>(2);
		unreel.add(toUnreelFrom);
		unreel.addAll(toUnreelFrom.getLines());
		return UnreelLine(unreel);
	}

	/**
	 * Attempts to create a new line that starts from the given list of Items
	 * ('unreeling'). The List must contain only one non-line Item. The non-line
	 * Item must already have at least one line to be unreeled from.
	 * 
	 * @param toUnreel
	 *            The List containing the Item that will be one end point of the
	 *            new line
	 * @return A List of the newly created Item and Line that unreel from the
	 *         Item in the given List, or null if this List cannot be unreeled
	 *         from.
	 */
	public static List<Item> UnreelLine(List<Item> toUnreel) {
		Item origEnd = null;
		// find the end being unreeled from
		for (Item item : toUnreel) {
			// we dont want to unreel anything other than lines
			if (!(item instanceof Dot || item instanceof Line)) {
				return null;
			}
			//find the dot to unreel from
			if (item instanceof Dot) {
				// if there are multiple ends in the list, return
				if (origEnd != null)
					return null;

				origEnd = item;
			}
		}

		// copy the original endpoint
		Item copy = origEnd.copy();
		copy.removeAllLines();
		copy.removeAllConstraints();

		int num = 0;

		//Mike: WHAT IS THIS CODE DOING?!!?
		for (Line line : origEnd.getLines())
			num = Math.min(num, line.getID());

		num--;

		// create a new line
		Line line = new Line(origEnd, copy, num);
		//copy.setFloating(true);
		origEnd.setArrowheadLength(0);
		//copy.setArrowheadLength(0);

		List<Item> toReturn = new LinkedList<Item>();
		toReturn.add(copy);
		toReturn.add(line);
		return toReturn;
	}

	/**
	 * Checks through all Lines and Dots on the current Frame to detect if any
	 * form an enclosure, which can then be used to manipulate items within the
	 * polygon. If an enclosure is found, then the dots will have their
	 * enclosure value set to true, and a List is created that contains all the
	 * Dots in the order they were processed. Actual calculation of the Polygon
	 * is done dynamically (to account for Dots being moved).
	 */
	public static void EnclosedCheck(List<Item> items) {
		_seen.clear();

		// get all dots on the Frame
		List<Dot> dots = new ArrayList<Dot>(0);
		for (Item i : items) {
			if (i instanceof Dot) {
				Dot d = (Dot) i;
				d.setEnclosedList(null);

				if (d.getLines().size() == 2)
					dots.add(d);
			}
		}

		// if there are no dots on the Frame, then there can't be an enclosure
		if (dots.size() == 0)
			return;

		//TODO optimise this code!!
		// iterate through all the dots
		for (Dot searchFor : dots) {
			_seen.clear();

			for (Line l : searchFor.getLines()) {
				_seen.add(l);
				if (traverse(searchFor, l.getOppositeEnd(searchFor))) {
					_path.add((Dot) l.getOppositeEnd(searchFor));

					for (Dot d : _path)
						d.setEnclosedList(_path);

					_path = new ArrayList<Dot>(0);

					break;
				}
			}
		}

	}

	private static List<Line> _seen = new ArrayList<Line>();

	private static List<Dot> _path = new ArrayList<Dot>();

	private static boolean traverse(Dot toFind, Item searchFrom) {
		if (toFind == null || searchFrom == null
				|| !(searchFrom instanceof Dot))
			return false;

		if (searchFrom.getLines().size() != 2)
			return false;

		if (toFind == searchFrom)
			return true;

		for (Line l : searchFrom.getLines()) {
			if (!(_seen.contains(l))) {
				_seen.add(l);
				if (traverse(toFind, l.getOppositeEnd(searchFrom))) {
					_path.add((Dot) l.getOppositeEnd(searchFrom));
					return true;
				}
			}

		}

		return false;
	}
}
